@@ -15,10 +15,29 @@
 ;;;;
 ;;;; See also x86-vm for commentary on signed vs unsigned.
 
+(define-alien-routine ("os_context_float_register_addr" context-float-register-addr)
+  (* unsigned) (context (* os-context-t)) (index int))
+
 (defun context-float-register (context index format &optional integer)
-  (declare (ignorable context index integer))
-  (warn "stub CONTEXT-FLOAT-REGISTER")
-  (coerce 0 format))
+  (declare (ignorable context index ))
+  (aver (not integer))
+  #+linux
+  (let ((sap (alien-sap (context-float-register-addr context index))))
+    (ecase format
+      (single-float
+       (sap-ref-single sap 0))
+      (double-float
+       (sap-ref-double sap 0))
+      (complex-single-float
+       (complex (sap-ref-single sap 0)
+                (sap-ref-single sap 4)))
+      (complex-double-float
+       (complex (sap-ref-double sap 0)
+                (sap-ref-double sap 8)))))
+  #-linux
+  (progn
+    (warn "stub CONTEXT-FLOAT-REGISTER")
+    (coerce 0 format)))
 
 (defun %set-context-float-register (context index format new-value)
   (declare (ignore context index))
