@@ -1463,7 +1463,7 @@ line break."
         (output-object object stream)
         (dx-let ((state (cons 0 stream)))
           (if obj-supplied-p
-              (with-circularity-detection (object stream)
+              (with-circularity-detection (object stream state)
                 (descend-into (stream)
                   (start-logical-block stream prefix per-line-p suffix)
                   (funcall proc object state stream)
