@@ -128,10 +128,12 @@
               (values drop-through target)
               (values target drop-through))
         (assemble ()
-          (flet ((check-widetag (target not-p)
+          (flet ((check-widetag (target not-p &optional (nil-in-other-pointers nil-in-other-pointers))
                    (unless (and value-tn-ref
                                 (= lowtag other-pointer-lowtag)
-                                (other-pointer-tn-ref-p value-tn-ref nil-in-other-pointers immediate-tested))
+                                (other-pointer-tn-ref-p value-tn-ref
+                                                        nil-in-other-pointers
+                                                        immediate-tested))
                      (multiple-value-bind (bit set) (tn-ref-lowtag-bit lowtag value-tn-ref
                                                                        nil-in-other-pointers immediate-tested)
                        (when (and set (not not-p))
@@ -142,7 +144,12 @@
                          (0
                           (inst tbnz* value bit target))
                          (t
-                          (%test-lowtag value widetag target not-p lowtag))))
+                          (cond ((other-pointer-tn-ref-p value-tn-ref t immediate-tested)
+                                 ;; It's either NIL or an other-pointer
+                                 (inst cmp value null-tn)
+                                 (inst b (if not-p :eq :ne) target))
+                                (t
+                                 (%test-lowtag value widetag target not-p lowtag))))))
                      t)))
             (cond
               ((and value-tn-ref
@@ -156,7 +163,7 @@
                                     thereis (if (consp header)
                                                 (<= (car header) widetag (cdr header))
                                                 (eql widetag header)))))))
-               (or (check-widetag target not-p)
+               (or (check-widetag target not-p nil)
                    (unless not-p
                      (inst b target))))
               (t
@@ -260,7 +267,7 @@
     not-target))
 
 (define-vop (signed-byte-64-p-move-to-word signed-byte-64-p)
-  (:results (r :scs (signed-reg)))
+  (:results (r :scs (signed-reg unsigned-reg)))
   (:result-types signed-num)
   (:translate)
   (:generator 10
@@ -424,7 +431,7 @@
     NOT-TARGET))
 
 (define-vop (unsigned-byte-64-p-move-to-word unsigned-byte-64-p)
-  (:results (r :scs (unsigned-reg)))
+  (:results (r :scs (signed-reg unsigned-reg)))
   (:result-types unsigned-num)
   (:translate)
   (:generator 10
@@ -490,8 +497,8 @@
 (define-vop (un/signed-byte-64-p-move-to-word signed-byte-64-p)
   ;; Ideally, this would use a single register but the rest of the stuff
   ;; depends on their storage class.
-  (:results (rs :scs (signed-reg))
-            (ru :scs (unsigned-reg)))
+  (:results (rs :scs (signed-reg unsigned-reg))
+            (ru :scs (signed-reg unsigned-reg)))
   (:info target not-p target-unsigned not-p-unsigned unsigned-fall-through)
   (:result-types signed-num unsigned-num)
   (:translate)
