@@ -60,7 +60,7 @@ arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
 {
     os_vm_address_t addr;
 
-#if defined(LISP_FEATURE_NETBSD) || defined(LISP_FEATURE_OPENBSD)
+#if defined(LISP_FEATURE_NETBSD) || defined(LISP_FEATURE_OPENBSD) || defined(LISP_FEATURE_FREEBSD)
     addr = (os_vm_address_t) (code->si_addr);
 #else
     addr = (os_vm_address_t) (*os_context_register_addr(context,PT_DAR));
@@ -554,7 +554,13 @@ sigtrap_handler(int signal, siginfo_t *siginfo, os_context_t *context)
         return;
     }
 
-    interrupt_handle_now(signal, (siginfo_t *)code, context);
+    if (signal == SIGILL) {
+        fake_foreign_function_call(context);
+        lose("Unhandled SIGILL at %p.", (void*)OS_CONTEXT_PC(context));
+    }
+    else {
+        interrupt_handle_now(signal, (siginfo_t *)siginfo, context);
+    }
 }
 
 
