@@ -16,6 +16,7 @@
 ;;;; more information.
 
 (cl:in-package :cl-user)
+(load "compiler-test-util.lisp")
 
 ;;; In sbcl-0.6.10, Douglas Brebner reported that (SETF EXTERN-ALIEN)
 ;;; was messed up so badly that trying to execute expressions like
@@ -204,15 +205,15 @@
    (eq :ok
        (handler-case
            (tagbody
-              (alien-funcall
-               (extern-alien "CallWindowProcW"
-                             (function unsigned-int
-                                       (* (function int)) unsigned-int
-                                       unsigned-int unsigned-int unsigned-int))
-               (alien-sap
-                (sb-alien::alien-callback (function unsigned-int)
-                                          #'(lambda () (go up))))
-               0 0 0 0)
+              (with-alien-callable ((callback unsigned-int ()
+                                      (go up)))
+                (alien-funcall
+                 (extern-alien "CallWindowProcW"
+                               (function unsigned-int
+                                         (* (function int)) unsigned-int
+                                         unsigned-int unsigned-int unsigned-int))
+                 callback
+                 0 0 0 0))
             up
               (funcall 0))
          (error ()
@@ -270,11 +271,9 @@
 (declaim (inline bug-316075))
 ;; KLUDGE: This win32 reader conditional masks a bug, but allows the
 ;; test to fail cleanly.
-#-win32
 (locally (declare (muffle-conditions style-warning))
   (sb-alien:define-alien-routine bug-316075 void (result char :out)))
-(with-test (:name :bug-316075 :fails-on :win32)
-  #+win32 (error "fail")
+(with-test (:name :bug-316075)
   (checked-compile '(lambda () (multiple-value-list (bug-316075)))))
 
 ;;; Bug #316325: "return values of alien calls assumed truncated to
@@ -329,7 +328,7 @@
         (values (alien-funcall sys-execv1 program argv))))
    :allow-notes nil))
 
-(with-test (:name :bug-721087 :fails-on :win32)
+(with-test (:name :bug-721087)
   (assert (typep nil '(alien c-string)))
   (assert (not (typep nil '(alien (c-string :not-null t)))))
   (assert (eq :ok
@@ -337,8 +336,8 @@
                   (posix-getenv nil)
                 (type-error (e)
                   (when (and (null (type-error-datum e))
-                             (equal (type-error-expected-type e)
-                                    '(alien (c-string :not-null t))))
+                             #-win32 (equal (type-error-expected-type e)
+                                            '(alien (c-string :not-null t))))
                     :ok))))))
 
 (with-test (:name :make-alien-string)
@@ -366,30 +365,6 @@
     (test nil)
     (test t)))
 
-;;; Skip for MSAN. Instead of returning 0, the intercepted malloc is configured
-;;; to cause process termination by default on failure to allocate memory.
-;;; Skip also for UBSAN which has a smaller ARRAY-TOTAL-SIZE-LIMIT
-;;; and so doesn't get ENOMEM.
-;;; Unfortunately, even without an intercepted malloc, and depending on SBCL build
-;;; parameters (notably GC card-size) you might not get a failure right away,
-;;; but instead suffer process death, or cause your kernel to churn for a while
-;;; as it looks for swap space and then decides to OOM-kill you. This will typically
-;;; occur when ARRAY-TOTAL-SIZE-LIMIT is "too small" to get instant failure.
-;;; Instead, your malloc() thinks the request is reasonable, and tries to fulfill it.
-;;; But we're constrained by the maximum BYTES argument to MAKE-%ALIEN which
-;;; is declared as INDEX even though we want to pass something ludicrously
-;;; big like half the maximum value of size_t.
-#+64-bit (unless (>= (integer-length array-total-size-limit) 45)
-           (push :skip-malloc-test *features*))
-(with-test (:name :malloc-failure ; for lp#891268
-                  :skipped-on (or :ubsan :msan :skip-malloc-test))
-  (assert (eq :enomem
-              (handler-case
-                  (loop repeat 128
-                        collect (sb-alien:make-alien char (1- array-total-size-limit)))
-                (storage-condition ()
-                  :enomem)))))
-
 (with-test (:name :bug-985505)
   ;; Check that correct octets are reported for a c-string-decoding error.
   (assert
@@ -569,6 +544,47 @@
 (with-test (:name :no-vector-sap-of-array-nil)
   (assert-error (sb-sys:vector-sap (opaque-identity (make-array 5 :element-type nil)))))
 
+(define-alien-variable internal-errors-enabled int)
+
+(with-test (:name :direct-and-indirect-deref
+                  :fails-on :interpreter)
+  (let ((fun (checked-compile
+              `(lambda (x y b)
+                 (declare (optimize speed (safety 0)))
+                 (declare (type (alien (* (* int))) x y))
+                 (let ((xx (deref x))
+                       (yy (deref y)))
+                   (values (deref xx) (deref yy) (let ((z (if b xx yy))) (deref z))))))))
+    (with-alien ((a (* int) (addr internal-errors-enabled)))
+      (multiple-value-bind (i j k) (funcall fun (addr a) (addr a) t)
+        (assert (= i j k 1)))
+      (multiple-value-bind (i j k) (funcall fun (addr a) (addr a) nil)
+        (assert (= i j k 1))))))
+
+;;; Permanent fnames are omitted from the list of referenced Lisp linkage table
+;;; indices in the code header. Prevent that behavior.
+(sb-int:encapsulate 'sb-int:permanent-fname-p 'test-shim #'sb-int:constantly-nil)
+(with-test (:name :string-passing-no-conversion :skipped-on (:not :sb-unicode))
+  (flet ((has-call (arg-type)
+           (let ((f (compile nil`(lambda (s)
+                                   (with-alien ((getenv (function unsigned utf8-string) :extern))
+                                     (alien-funcall getenv (the ,arg-type s)))))))
+             (find 'sb-alien::string-to-c-string (ctu:find-named-callees f)))))
+    ;; Positive assertion that passing STRING may (in theory) do a conversion
+    (assert (has-call 'string))
+    ;; Negative assertion that passing SIMPLE-BASE-STRING will never do a conversion
+    (assert (not (has-call 'simple-base-string)))))
+
+(with-test (:name :not-quite-literal-alien-name :skipped-on (:not :unix))
+  ;; There are valid reasons for the first argument to EXTERN-ALIEN to be an expression
+  ;; producing a constant string such as through a global constant or a macro that selects
+  ;; a name based on environmental aspects such as compilation mode and/or foreign toolchain.
+  (let ((f (compile nil
+            '(lambda (s)
+              (macrolet ((something () "getenv"))
+                (alien-funcall (extern-alien (something) (function c-string c-string)) s))))))
+    (assert (string= (funcall f "SBCL_HOME") (sb-ext:posix-getenv "SBCL_HOME")))))
+
 (cl:in-package "SB-KERNEL")
 (test-util:with-test (:name :hash-consing)
   (assert (eq (parse-alien-type '(integer 9) nil)
