@@ -66,7 +66,8 @@
   (checked-compile-and-assert
       ()
       '(lambda (n) (coerce n 'single-float))
-    (((expt 10 1000)) (condition 'floating-point-overflow))))
+    (((expt 10 1000)) #+no-float-traps sb-ext:single-float-positive-infinity
+                      #-no-float-traps (condition 'floating-point-overflow))))
 
 (defun are-we-getting-ash-right (x y)
   (declare (optimize speed)
@@ -417,7 +418,7 @@
 
 (with-test (:name :signed-byte-8-p-unsigned
                   ;; these lack the necessary RANGE<= vop
-                  :fails-on (:or :mips :ppc :ppc64 :sparc :riscv))
+                  :fails-on (:or :mips :ppc :ppc64 :sparc :riscv :loongarch64))
   (checked-compile
    `(lambda (a)
       (declare (type (simple-array sb-vm:word (*)) a)
@@ -518,6 +519,7 @@
       `(lambda (a)
          (declare (bit a))
          (/ 1 a))
+    (:return-type (values (eql 1) &optional))
     ((1) 1)
     ((0) (condition 'division-by-zero)))
   (checked-compile-and-assert
@@ -525,7 +527,30 @@
       `(lambda (a)
          (declare (bit a))
          (= (/ 5 a) 5))
+    (:return-type (values (eql t) &optional))
     ((1) t)
+    ((0) (condition 'division-by-zero)))
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (a)
+         (declare (integer a))
+         (/ 0 a))
+    (:return-type (values (eql 0) &optional))
+    ((1) 0)
+    ((0) (condition 'division-by-zero)))
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (a)
+         (declare (integer a))
+         (floor 0 a))
+    (:return-type (values (eql 0) (eql 0) &optional))
+    ((1) (values 0 0))
+    ((0) (condition 'division-by-zero)))
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (a)
+         (floor 0 a))
+    ((1.0) (values 0 0.0))
     ((0) (condition 'division-by-zero))))
 
 (with-test (:name :dpb-size-overflow)
@@ -568,14 +593,19 @@
 (with-test (:name :log-integer-derive-type)
   (assert-type
    (lambda (x)
-     (declare ((integer 0) x))
+     (declare ((integer 1) x))
      (log x))
    (single-float 0.0))
+  (assert-type
+   (lambda (x)
+     (declare ((integer 0) x))
+     (log x))
+   single-float)
   (assert-type
    (lambda (x)
      (declare (integer x))
      (log x))
-   (or (complex single-float) (single-float 0.0))))
+   (or (complex single-float) single-float)))
 
 (with-test (:name :floor-derive-type)
   (assert-type
@@ -586,7 +616,7 @@
    (values (integer -255 0) (integer -9 0) &optional)))
 
 (with-test (:name :logbitp-on-integers
-            :fails-on :arm)
+            :fails-on (or :ppc :ppc64 :arm :riscv :loongarch64 :mips :sparc))
   (assert (not (ctu:ir1-named-calls `(lambda (x)
                                        (logbitp 20 x))))))
 (with-test (:name :bt-negative-bit)
@@ -766,7 +796,8 @@
       n))
    (unsigned-byte 54)))
 
-(with-test (:name :ldb-computed-posn)
+(with-test (:name :ldb-computed-posn
+            :fails-on (or :ppc64 :riscv :loongarch64))
   (assert-type
    (lambda (x y)
      (ldb (byte y (- 32 y)) x))
@@ -820,7 +851,7 @@
    ((300 0) 303)))
 
 (with-test (:name :truncate-unknown-integer
-            :fails-on :arm)
+            :fails-on (or :arm :riscv :loongarch64))
   (assert (not (ctu:ir1-named-calls `(lambda (x)
                                        (declare (integer x))
                                        (values (the (signed-byte 25) (floor x 2)))))))
@@ -841,7 +872,7 @@
                                        (values (the (signed-byte 25) (ceiling x 2))))))))
 
 (with-test (:name :word-floor-ceiling
-            :fails-on :arm)
+            :fails-on (or :ppc :arm :riscv :loongarch64 :sparc))
   (assert (not (ctu:ir1-named-calls `(lambda (x y)
                                        (declare (sb-vm:signed-word x y))
                                        (floor x y)))))
@@ -1297,7 +1328,8 @@
      (ash x y))
    (integer * 0)))
 
-(with-test (:name :ash-overflow)
+(with-test (:name :ash-overflow
+            :fails-on (or :sparc :ppc :ppc64))
   (checked-compile-and-assert
       ()
       `(lambda (a)
@@ -1502,7 +1534,12 @@
         `(lambda (a b)
            (declare (integer b))
            (- (- (+ a 5)) b))
-      ((-5.0 0) -0.0))))
+      ((-5.0 0) -0.0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a)
+           (cos (- a)))
+      ((#C(0 2)) (cos #C(0 -2))))))
 
 (with-test (:name :remove-negate.2)
   (flet ((test (form count)
@@ -1533,7 +1570,53 @@
           0)
     (test `(lambda (x)
              (- (lognot x)))
-          0)))
+          0)
+    (test `(lambda (x)
+             (- (the integer (- x))))
+          0)
+    (test `(lambda (a)
+             (- (truncate a)))
+          0)
+    (test `(lambda (a)
+             (declare (integer a))
+             (- (ash (- a) -2)))
+          0)
+    (checked-compile-and-assert
+        ()
+        `(lambda (m a)
+           (declare (integer a))
+           (abs (if m
+                    (ash (abs a) -5)
+                    -9)))
+      ((t 63) 1)
+      ((nil 1) 9))
+    (checked-compile-and-assert
+        ()
+        `(lambda (r)
+           (declare (rational r))
+           (values (fceiling (- r) 2)))
+      ((0) 0.0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (r)
+           (- (- r) 5))
+      ((#C(.0 0)) #C(-5 -0.0)))
+    (checked-compile-and-assert
+        ()
+        `(lambda (c)
+           (- (* #C(4.0 3) c)))
+      ((#C(-0.0 0)) #C(0 -0.0)))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b)
+           (declare ((complex float) a b))
+           (- (* (- a) b)))
+      ((#C(-4.0 -3) #C(-0.0 0)) #C(0 -0.0)))
+    (checked-compile-and-assert
+        ()
+        `(lambda (f i)
+           (- (* f (truncate i 20))))
+      ((3.0 1) -0.0))))
 
 (with-test (:name :abs-match)
   (flet ((test (form count)
@@ -1806,3 +1889,544 @@
          (logand (sb-c::mask-signed-field sb-vm:n-fixnum-bits n) most-positive-word))
     (((* most-positive-fixnum 2))
      (logand -2 most-positive-word))))
+
+(with-test (:name :logand-cut-constants)
+  ;; (assert (= (count-if
+  ;;             (lambda (c)
+  ;;               (member c '(logand logtest)))
+  ;;             (ctu:ir1-named-calls `(lambda (x m)
+  ;;                                     (declare ((unsigned-byte 32) x)
+  ;;                                              ((unsigned-byte 8) m))
+  ;;                                     (logtest m (logand x #xFF)))
+  ;;                                  nil))
+  ;;            1))
+  (assert (= (count 'logand
+                    (ctu:ir1-named-calls `(lambda (m)
+                                            (declare ((unsigned-byte 8) m))
+                                            (logand m #xFFFF))
+                                         nil))
+             0))
+  (assert (= (count 'sb-c::mask-signed-field
+                    (ctu:ir1-named-calls `(lambda (n m)
+                                            (declare ((unsigned-byte 64) n)
+                                                     ((unsigned-byte 8) m))
+                                            (logand m (sb-c::mask-signed-field 63 n)))
+                                         nil))
+             0))
+  (assert (= (count 'logior
+                    (ctu:ir1-named-calls `(lambda (n)
+                                            (logand #xF0 (logior n 1)))
+                                         nil))
+             0))
+  (assert (= (count 'logxor
+                    (ctu:ir1-named-calls `(lambda (n)
+                                            (declare ((unsigned-byte 32) n))
+                                            (logand #xF (logxor n #xF0)))
+                                         nil))
+             0))
+  (assert-type
+   (lambda (n)
+     (logand #xFF (logior n #xFF)))
+   (eql #xFF))
+  (checked-compile-and-assert
+      ()
+      `(lambda (b d)
+         (declare ((unsigned-byte 32) b d))
+         (logand (the bit b)
+                 (logior d 1)))
+    ((0 3) 0)
+    ((1 3) 1))
+  (checked-compile-and-assert
+      ()
+      `(lambda (d)
+         (logand
+          73786976294838206461
+          (logior
+           (the (integer -3 -1) d)
+           -288230376562614601)))
+    ((-1) 73786976294838206461)
+    ((-3) 73786976294838206461))
+  (checked-compile-and-assert
+      ()
+      `(lambda (s)
+         (logand (logior -10 s) 29953653503380140701))
+    ((0) 29953653503380140692)
+    ((1) 29953653503380140693)))
+
+(with-test (:name :logand-cut-constants.2
+            :fails-on (or :arm :mips :ppc :sparc))
+  (assert (= (count-if (lambda (c)
+                         (member c '(logand sb-kernel:two-arg-and)))
+                       (ctu:ir1-named-calls `(lambda (n m)
+                                               (declare ((unsigned-byte 64) n)
+                                                        ((unsigned-byte 8) m))
+                                               (logand m (logand n #xFFFF)))
+                                            nil))
+             1))
+  #-arm
+  (assert (= (count-if (lambda (c)
+                         (member c '(logand sb-kernel:two-arg-and)))
+                       (ctu:ir1-named-calls `(lambda (x m)
+                                               (declare ((unsigned-byte 32) x)
+                                                        ((unsigned-byte 8) m))
+                                               (logand m (logand x #xFF)))
+                                            nil))
+             1)))
+
+(with-test (:name :lognot)
+  (assert (= (count 'lognot
+                    (ctu:ir1-named-calls `(lambda (x)
+                                            (declare (integer x))
+                                            (lognot (lognot x)))
+                                         nil))
+             0)))
+
+(with-test (:name :constant-association
+            :fails-on :sbcl)
+  (assert (= (count-if (lambda (c)
+                         (member c '(+ sb-kernel:two-arg-+)))
+                       (ctu:ir1-named-calls
+                        `(lambda (x)
+                           (+ (the integer (1+ x)) 20))
+                        nil))
+             1))
+  (assert (= (count-if (lambda (c)
+                         (member c '(* sb-kernel:two-arg-*)))
+                       (ctu:ir1-named-calls
+                        `(lambda (x)
+                           (* (the integer (* x 3)) 5))
+                        nil))
+             1)))
+
+(with-test (:name :0/0)
+  (assert-error
+      (/ (opaque-identity 0) (opaque-identity 0))
+      division-by-zero))
+
+(with-test (:name :assoc-*-const)
+  (flet ((test (names form count)
+           (assert (= (count-if (lambda (c)
+                                  (member c names))
+                                (ctu:ir1-named-calls form nil))
+                      count))))
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a b)
+             (declare (rational a b))
+             (* (* a (* b 3)) 5))
+          2)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a b)
+             (declare (rational a b))
+             (* (/ (* b 5) a) 6))
+          1)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a)
+             (declare (rational a))
+             (* (/ (* a 5) 4) 6))
+          1)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a)
+             (declare (rational a))
+             (* (abs (* a 5)) 6))
+          1)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a b)
+             (declare (rational a b))
+             (* (* a 3) (* b 5)))
+          2)
+    (test '(/ sb-kernel:two-arg-/)
+          `(lambda (a)
+             (declare (rational a))
+             (* (/ a 3) 6))
+          0)
+    (test '(ash)
+          `(lambda (a)
+             (* (ash a 3) 5))
+          0)
+    ;; (test '(* sb-kernel:two-arg-*)
+    ;;       `(lambda (a s)
+    ;;          (declare (unsigned-byte s))
+    ;;          (* (ash (* a 3) s) 5))
+    ;;       1)
+    ;; (test '(* sb-kernel:two-arg-* ash)
+    ;;       `(lambda (a)
+    ;;          (ash (* a 3) 2))
+    ;;       1)
+    (test '(truncate sb-kernel::truncate1)
+          `(lambda (a)
+             (declare (integer a))
+             (values (truncate (* a 9) 3)))
+          0)
+    (test '(floor sb-kernel::floor1)
+          `(lambda (a)
+             (declare (integer a))
+             (values (floor (* a 9) 3)))
+          0)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a)
+             (declare (rational a))
+             (values (truncate (* a 3) 9)))
+          0)
+    (assert-type
+     (lambda (a)
+       (declare (integer a))
+       (rem (* a 20) 10))
+     (eql 0))
+    (test '(truncate sb-kernel::truncate1)
+          `(lambda (a)
+             (declare (integer a))
+             (values (truncate (truncate a 10) 20)))
+          1)
+    (test '(floor sb-kernel::floor1)
+          `(lambda (a)
+             (declare (integer a))
+             (values (floor (floor a 10) 20)))
+          1)
+    (checked-compile-and-assert
+        ()
+        `(lambda (v)
+           (declare (integer v))
+           (values (round (round v -94) 40)))
+      ((2396371438047407363) -637332829267928))
+    (checked-compile-and-assert
+        ()
+        `(lambda (v)
+           (declare (integer v))
+           (values (ceiling (ceiling v 7) -3)))
+      ((15) -1))
+    (checked-compile-and-assert
+        ()
+        `(lambda (v)
+           (declare (integer v))
+           (values (floor (abs (floor v 5)) 4)))
+      ((-19) 1))
+    (checked-compile-and-assert
+        ()
+        `(lambda (v)
+           (declare (integer v))
+           (values (ceiling (abs (ceiling v -5)) 10)))
+      ((1) 0))
+    (test '(floor sb-kernel::floor1)
+          `(lambda (a)
+             (declare (unsigned-byte a))
+             (values (floor (abs (floor a -10)) 20)))
+          1)
+    (test '(ceiling sb-kernel::ceiling1)
+          `(lambda (a)
+             (declare (unsigned-byte a))
+             (values (ceiling (abs (ceiling a 10)) 20)))
+          1)))
+
+(with-test (:name :assoc-*-const.2)
+  (flet ((test (names form count)
+           (assert (= (count-if (lambda (c)
+                                  (member c names))
+                                (ctu:ir1-named-calls form nil))
+                      count))))
+    (test '(truncate sb-kernel::truncate1 ash sb-c::ash-right)
+          `(lambda (a)
+             (declare (unsigned-byte a))
+             (values (truncate (ash a -2) 5)))
+          1)
+    (test '(floor sb-kernel::floor1 ash sb-c::ash-right)
+          `(lambda (a)
+             (values (floor (ash a -2) 5)))
+          1)
+    (test '(ash sb-c::ash-right)
+          `(lambda (n)
+             (declare (integer n))
+             (ash (floor n 5) -2))
+          0)
+    (test '(ash sb-c::ash-right)
+          `(lambda (n)
+             (declare (unsigned-byte n))
+             (ash (truncate n 5) -2))
+          0)
+    (test '(truncate sb-kernel::truncate1)
+          `(lambda (a)
+             (declare (integer a))
+             (values (truncate (truncate 9 a) 3)))
+          1)
+    (checked-compile-and-assert
+        ()
+        `(lambda (v)
+           (declare (integer v))
+           (values (truncate (abs (truncate 9 v)) -3)))
+      ((2) -1)
+      ((-10) 0))))
+
+(with-test (:name :assoc-*-const.3)
+  (flet ((test (names form count)
+           (assert (= (count-if (lambda (c)
+                                  (member c names))
+                                (ctu:ir1-named-calls form nil))
+                      count))))
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a)
+             (declare (rational a))
+             (* (+ (* a 3) 3) 5))
+          1)
+    (test '(* sb-kernel:two-arg-*)
+          `(lambda (a)
+             (declare (rational a))
+             (* (abs (- (* a 3) 3)) 5))
+          1)
+    (checked-compile-and-assert
+        ()
+        `(lambda (a)
+           (declare (integer a))
+           (* (+ (abs (* -3 a)) 3) -5))
+      ((5) -90))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a)
+           (declare (integer a))
+           (values (floor (abs (floor 10 a)) 5)))
+      ((-9) 0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (p)
+           (declare (integer p))
+           (values (ceiling (ceiling (* 10 p) -31)
+                            41/50)))
+      ((-97) 40))
+    (checked-compile-and-assert
+        ()
+        `(lambda (n)
+           (declare (integer n))
+           (values (truncate (truncate n 190) 16/25)))
+      ((46945) 385))
+    (checked-compile-and-assert
+        ()
+        `(lambda (n)
+           (declare (rational n))
+           (values (truncate (truncate 4 n) 3)))
+      ((1/3) 4))
+    (checked-compile-and-assert
+        ()
+        `(lambda (n)
+           (declare (integer n))
+           (* (/ 23 (* 37 n)) -12))
+      ((2) -138/37))
+    (checked-compile-and-assert
+        (:optimize :safe)
+        `(lambda (n)
+           (* (the integer (* #c(1 2) n)) 3))
+      ((#c(1 -2)) 15)
+      ((2) (condition 'type-error)))
+    (checked-compile-and-assert
+        (:optimize :safe)
+        `(lambda (n)
+           (* (the integer (* n 4)) 3))
+      ((1/2) 6))
+    (checked-compile-and-assert
+        (:optimize :safe)
+        `(lambda (n)
+           (* (the integer (* #c(1 2) n)) 3))
+      ((#c(1 -2)) 15)
+      ((2) (condition 'type-error)))
+    (checked-compile-and-assert
+        (:optimize :safe)
+        `(lambda (n)
+           (* (the integer (* n 0)) 3))
+      ((#c(1 -2)) 0))))
+
+(with-test (:name :logtest
+            :fails-on :ppc64)
+  (when (ctu:vop-existsp 'logtest)
+    (assert (= (count 'logtest
+                      (ctu:ir1-named-calls `(lambda (a)
+                                              (zerop (logand a 1)))
+                                           nil))
+               1))
+    (assert (= (count 'logtest
+                      (ctu:ir1-named-calls `(lambda (a)
+                                              (declare (fixnum a))
+                                              (plusp (logand a 6)))
+                                           nil))
+               1)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (p1)
+         (declare (type (integer -845794755782386 1048630) p1))
+         (logtest p1 12115639945877374832))
+    ((-845794755782386) t)
+    ((145551) nil))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (logtest a (the (unsigned-byte 30) a)))
+    ((8) t)
+    ((0) nil)))
+
+(with-test (:name :logtest-integer-fixnum
+            :fails-on (or :arm :ppc64 :ppc :riscv :loongarch64 :mips :sparc))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (integer x)
+                                                (fixnum y))
+                                       (logtest x y))))))
+
+(with-test (:name :ash-mod)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare ((integer -3 3) b))
+         (logand #xF (+ (ash a b) 1)))
+    ((-1 3) 9)
+    ((-1 -3) 0)
+    ((1 -3) 1))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare (fixnum a b))
+         (logand most-positive-word (ash a b)))
+    ((1 2) 4)
+    ((10 -2) 2)
+    ((1 sb-vm:n-word-bits) 0)
+    ((-1 sb-vm:n-word-bits) 0)
+    ((1 (- sb-vm:n-word-bits)) 0)
+    ((-1 (- sb-vm:n-word-bits)) most-positive-word))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare (fixnum a b))
+         (logand most-positive-word (ash a (- b))))
+    ((1 -2) 4)
+    ((10 2) 2)
+    ((1 (- sb-vm:n-word-bits)) 0)
+    ((-1 (- sb-vm:n-word-bits)) 0)
+    ((1 sb-vm:n-word-bits) 0)
+    ((-1 sb-vm:n-word-bits) most-positive-word)))
+
+(with-test (:name :division-float-0-type)
+  (assert-type
+     (lambda (x y)
+       (declare ((real 0) x y))
+       (/ x y))
+     (real 0)))
+
+(with-test (:name :floor/ceiling-to-truncate)
+  (assert (= (count-if (lambda (c)
+                         (member c '(truncate)))
+                       (ctu:ir1-named-calls
+                        `(lambda (x y)
+                           (declare ((real 0 *) x y))
+                           (floor x y))
+                        nil))
+             1))
+  (assert (= (count-if (lambda (c)
+                         (member c '(truncate)))
+                       (ctu:ir1-named-calls
+                        `(lambda (x y)
+                           (declare ((real * 0) x y))
+                           (floor x y))
+                        nil))
+             1))
+  (assert (= (count-if (lambda (c)
+                         (member c '(truncate)))
+                       (ctu:ir1-named-calls
+                        `(lambda (x y)
+                           (declare ((real * 0) x)
+                                    ((real 0 *) y))
+                           (ceiling x y))
+                        nil))
+             1)))
+(with-test (:name :range<-empty)
+  (if (ctu:vop-existsp 'sb-kernel:range<<=)
+      (assert-type
+       (lambda (v)
+         (declare ((integer -50 0) v))
+         (> v
+            (if (> v -30)
+                1
+                -10)))
+       null)
+      (assert-type
+       (lambda (v)
+         (declare ((integer -50 0) v))
+         (> v
+            (if (> v -30)
+                1
+                -10)))
+       boolean)))
+
+(with-test (:name :ash-left-bignum)
+  (checked-compile-and-assert
+      ()
+      `(lambda (b s)
+         (declare (bignum b)
+                  ((member -40 25) s))
+         (logand #xffffffffffff (ash b s)))
+    ((-1422907942930057474717222 25) 75401170780160)
+    ((-1422907942930057474717222 -40) 280180849475450)))
+
+(with-test (:name :truncate-ratios-by-float)
+  (assert-type
+   (lambda (r)
+     (declare ((rational 9241088767394112609508907202529414093/970467704224781278
+                         9241088767394112609508907202529414093/970467704224781277)
+               r))
+     (truncate r 1.0))
+   (values (eql 9522303959679631360) (eql 0.0) &optional))
+  (assert-type
+   (lambda (x)
+     (declare ((single-float 0.0 0.0) x))
+     (nth-value 1 (truncate x)))
+   (single-float 0.0 0.0))
+  (assert-type
+   (lambda (v1)
+     (declare ((rational (96106637441700886132) (96106637441700886133)) v1))
+     (truncate v1 1.0))
+   (values (integer 96106640126225940480 96106640126225940480) (member 0.0) &optional)))
+
+(with-test (:name :complex-division-minus-zero)
+  (loop for x in (opaque-identity '(0 0.0 #c(0.0 0.0)))
+        do (assert (eql (/ x #c(-1.0 3))
+                        (opaque-identity #C(0.0 -0.0)))))
+  (loop for x in (opaque-identity '(2 2.0 #c(2.0 0.0)))
+        do (assert (eql (/ x #c(0.0 1.0))
+                        (opaque-identity #C(0.0 -2.0)))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (f c)
+         (declare (single-float f)
+                  ((complex single-float) c))
+         (/ f c))
+    ((-2.0 #c(0.0 1.0)) #C(0.0 2.0))
+    ((0.0 #c(1 -3.0)) (/ 0 #c(1 -3.0))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (f c)
+         (declare (real f)
+                  ((complex single-float) c))
+         (/ f c))
+    ((-2.0 #c(0.0 1.0)) #C(0.0 2.0))
+    ((0.0 #c(1 -3.0)) (/ 0 #c(1 -3.0)))
+    ((0  #C(3.0 -4)) (/ 0.0 #C(3.0 -4))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (c)
+         (- (/ -3.0 c)))
+    ((#c(0 3.0)) (- (opaque-identity (/ (opaque-identity -3.0)
+                                        (opaque-identity #c(0 3.0)))))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare ((complex float) a b))
+         (/ (- a) (- b)))
+    ((#c(0.0 3.0) #c(0.0 3.0))
+     (/ (opaque-identity (- #c(0.0 3.0)))
+        (opaque-identity (- #c(0.0 3.0))))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare (double-float n))
+         (/ n #C(1.0 2.0)))
+    ((5d0) #C(1.0d0 -2.0d0))))
+
+(with-test (:name :truncate+/+complex)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (values (truncate (/ a b))))
+    ((#C(3 6) #C(1 2)) 3)))
