@@ -479,7 +479,8 @@
                  (the integer (the (real 5) x))))
              1)))
 
-(with-test (:name :sign-extend)
+(with-test (:name :sign-extend
+            :fails-on (or :riscv :loongarch64))
   (assert (= (count 'sb-c::mask-signed-field
                     (ir-calls
                      `(lambda (a)
@@ -533,7 +534,8 @@
                         (truly-the fixnum (funcall (the function x)))))
     (assert (node-tail-p combination))))
 
-(with-test (:name :evenp+arithmetic)
+(with-test (:name :evenp+arithmetic
+            :fails-on (or :riscv :loongarch64 :ppc64 :ppc))
   (assert (not (ir-full-calls `(lambda (x)
                                  (evenp (+ x 3))))))
   (assert (not (ir-full-calls `(lambda (x)
@@ -584,7 +586,8 @@
                                  (lambda (x) (1+ x)))
                             n))))))
 
-(with-test (:name :truncate-signed-word-error)
+(with-test (:name :truncate-signed-word-error
+            :fails-on (or :ppc64 :riscv :loongarch64))
   (assert (not (find 'sb-vm::move-from-signed
                      (ir2-vops '(lambda (x d)
                                  (declare ((signed-byte 64) x d))
