@@ -826,7 +826,8 @@
              0)))
 
 
-(with-test (:name :bounds-check-min-length)
+(with-test (:name :bounds-check-min-length
+            :fails-on (or :ppc :ppc64 :riscv :loongarch64 :sparc :mips))
   (assert (= (count 'sb-kernel:%check-bound
                     (ctu:ir1-named-calls
                      `(lambda (x v)
@@ -1909,7 +1910,7 @@
    (integer -6176139011 -222)))
 
 (with-test (:name :ignore-delays
-            :fails-on :arm)
+            :fails-on (or :arm :riscv :loongarch64 :ppc :ppc64 :mips :sparc))
   (assert-type
    (lambda (x)
      (declare (optimize debug))
@@ -2029,3 +2030,70 @@
    (lambda (x)
      (when (typep (the (or float (rational (-1/2) (1))) (nth-value 1 (truncate x 1))) 'float) x))
    (or float null)))
+
+(with-test (:name :div-by-zero)
+  (assert-type
+   (lambda (x y)
+     (declare (float y))
+     (floor x y)
+     y)
+   float)
+  (assert-type
+   (lambda (x y)
+     (declare (rational x y))
+     (floor x y)
+     y)
+   (and rational (not (eql 0)))))
+
+(with-test (:name :complex-into-real)
+  (assert-type
+   (lambda (c)
+     (if (> (- c c) 10)
+         c
+         (error "")))
+   number)
+  (assert-type
+   (lambda (c)
+     (if (> (+ c c) 10)
+         c
+         (error "")))
+   real)
+  (assert-type
+   (lambda (c)
+     (if (> (* c c) 10)
+         c
+         (error "")))
+   number)
+  (assert-type
+   (lambda (c n)
+     (declare (integer n))
+     (if (> (* c n) 10)
+         c
+         (error "")))
+   number)
+  (assert-type
+   (lambda (c n)
+     (declare ((integer 1) n))
+     (if (> (* c n) 10)
+         c
+         (error "")))
+   real)
+  (assert-type
+   (lambda (n c)
+     (declare (integer n))
+     (if (realp (/ n c))
+         c
+         (error "")))
+   number)
+  (assert-type
+   (lambda (n c)
+     (declare ((integer 1) n))
+     (if (realp (/ n c))
+         c
+         (error "")))
+   real)
+  (assert-type
+   (lambda (n)
+     (the integer (* n n))
+     n)
+   (or integer (complex rational))))
