@@ -308,9 +308,9 @@
       (assert (eq (read-char s nil s) s)))))
 
 ;; Output routines must return the written element
-(with-test (:name :output-routine-retval :skipped-on :win32)
+(with-test (:name :output-routine-retval)
   (dolist (x sb-impl::*output-routines*)
-    (with-open-file (f "/dev/null" :direction :output :if-exists :overwrite)
+    (with-open-file (f #-win32 "/dev/null" #+win32 "nul" :direction :output :if-exists :overwrite)
       (let ((arg (if (eq (car x) 'character) #\z 99))
             (fun (symbol-function (third x))))
         (assert (eql arg (funcall fun f arg)))))))
