@@ -19,6 +19,16 @@ primary_c_source=`pwd`/foreigntest.c
 . ./expect.sh
 . ./subr.sh
 
+run_sbcl <<EOF
+#+sparc (exit :code 2) ;skip
+(exit :code 0)
+EOF
+status=$?
+if [ $status != 0 ]; then # test can't be executed
+    # we don't have a way to exit shell tests with "inapplicable" as the result
+    exit $EXIT_TEST_WIN
+fi
+
 use_test_subdirectory
 
 echo //entering foreign.test.sh
