@@ -38,7 +38,7 @@
 
 ;; Regression from R/O space change
 (with-test (:name (apropos-list :smoke-test))
-  (assert (plusp (length (apropos-list "required-foreign")))))
+  (assert (plusp (length (apropos-list "**finalizer-store**")))))
 
 ;;; APROPOS should accept a package designator, not just a package, and
 ;;; furthermore do the right thing when it gets a package designator.
@@ -190,7 +190,8 @@
           (string2 (disassembly-text '(lambda (y) (car y)))))
       (assert (string= string1 string2)))))
 
-(with-test (:name :disassemble-assembly-routine)
+(with-test (:name :disassemble-assembly-routine
+            :fails-on :riscv)
   (disassemble sb-fasl:*assembler-routines* :stream (make-broadcast-stream)))
 
 (with-test (:name (sb-ext:assert-version->= :ok))
