@@ -59,7 +59,6 @@
 
 ;;; Bug 51b. (try to throw READER-ERRORs when the reader encounters
 ;;; dubious input)
-(assert-error (read-from-string "1e1000") reader-error)
 (assert-error (read-from-string "1/0") reader-error)
 
 ;;; Bug reported by Antonio Martinez on comp.lang.lisp 2003-02-03 in
@@ -203,8 +202,20 @@
     (list :metavar (read stream t nil t)))
   (assert (equal (read-from-string "$(x)") '(:metavar x)))
 
+  ;; What happens if the fun-designator is NIL? It used to remove the macro.
+  ;; I don't see evidence in the CLHS that this is correct, so now it implies
+  ;; that the character can't be read except where it behaves as a constituent.
+  ;; There is divided opinion amongst FOSS Lisp implementations I tested.
+  ;; Choice (1) fail at the call to SET-MACRO-CHARACTER
+  ;;   CLISP: (set-macro-character #\$ nil) => "undefined function NIL"
+  ;; Choice (2) remove the macro, causing #\$ become normal syntax.
+  ;;   ABCL and Clozure both do this
+  ;; Choice (3) fail when reading the character #\$
+  ;;   GCL: "Cell error on NIL: Undefined function"
+  ;;   ECL: "The function NIL is undefined."
+  ;;   Clasp: "The function NIL is undefined."
   (set-macro-character #\$ nil) ; 'NIL never designates a function
-  (assert (eq (read-from-string "$") '$))
+  (assert-error (read-from-string "$"))
 
   ;; Do not accept extended-function-designators.
   ;; (circumlocute to prevent a compile-time error)
@@ -368,7 +379,8 @@
   (assert-error (read-from-string "#S(NODE :NEXT #(#S(NODE :NEXT NIL)))"))
   (assert-error (read-from-string "#S(NODE :NEXT #S(NODE :NEXT 1))")))
 
-(with-test (:name (:sharp=-typed-slot :circular :no-error))
+(with-test (:name (:sharp=-typed-slot :circular :no-error)
+            :fails-on :sparc)
   (let ((circ (read-from-string "#1=#S(NODE :NEXT #1#)")))
     (assert (eql (node-next circ) circ))))
 (with-test (:name (:sharp=-typed-slot :circular error))
@@ -387,7 +399,8 @@
   (assert-error (read-from-string "#S(NODE :LISTNEXT #(#S(NODE :LISTNEXT NIL)))"))
   (assert-error (read-from-string "#S(NODE :LISTNEXT (#S(NODE :LISTNEXT 1)))")))
 
-(with-test (:name (:sharp=-cons-typed-slot :circular :no-error))
+(with-test (:name (:sharp=-cons-typed-slot :circular :no-error)
+            :fails-on :sparc)
   (let ((circ (read-from-string "#1=#S(NODE :LISTNEXT (#1#))")))
     (assert (eql (car (node-listnext circ)) circ))))
 (with-test (:name (:sharp=-cons-typed-slot :circular error))
@@ -401,7 +414,8 @@
   (assert-error (read-from-string "#S(NODE :CONSCONS 1)"))
   (assert-error (read-from-string "#S(NODE :CONSCONS (1))")))
 
-(with-test (:name (:sharp=-cons-typed-cons-slot :circular :no-error))
+(with-test (:name (:sharp=-cons-typed-cons-slot :circular :no-error)
+            :fails-on :sparc)
   (let* ((circ (car (read-from-string "#1=(#S(NODE :CONSCONS #1#) . #1#)")))
          (conscons (node-conscons circ)))
     (assert (eql (car conscons) circ))
@@ -435,7 +449,8 @@
     (dotimes (i 5)
       (assert (eql (aref displacement i) array)))))
 
-(with-test (:name (:sharp= :circular-mismatch))
+(with-test (:name (:sharp= :circular-mismatch)
+            :fails-on :sparc)
   (assert-error
       (read-from-string "#S(NODE :NEXT (#1=#S(NODE :NEXT #1#)))")
       type-error))
