@@ -338,18 +338,14 @@
       (set-syntax-from-char #\! #\! rt)
       (assert (eq '!! (maybe-bang))))))
 
-(with-test (:name :read-in-package-syntax :skipped-on :sb-devel)
+(with-test (:name :read-in-package-syntax)
   (assert (equal '(sb-c::a (sb-kernel::x sb-kernel::y) sb-c::b)
                  (read-from-string "sb-c::(a sb-kernel::(x y) b)")))
   (assert (equal '(cl-user::yes-this-is-sbcl)
-                 (read-from-string "cl-user::(#+sbcl yes-this-is-sbcl)")))
-  (assert (eq :violated!
-              (handler-case
-                  (read-from-string "cl::'foo")
-                (package-lock-violation ()
-                  :violated!)))))
+                 (read-from-string "cl-user::(#+sbcl yes-this-is-sbcl)"))))
 
-(with-test (:name :bug-309070)
+(with-test (:name :bug-309070
+            :fails-on :no-float-traps)
   (with-timeout 10
     (assert-error (read-from-string "10e10000000000000000000")
                   sb-kernel:reader-impossible-number-error)))
@@ -386,7 +382,8 @@
    (read-from-string string)))
 (compile 'test-reader-consing)
 
-(with-test (:name :read-does-not-cons-per-se)
+(with-test (:name :read-does-not-cons-per-se
+            :fails-on (or :mips :sparc))
     ;; These each used to produce at least 20 MB of garbage,
     ;; a result of using 128-character (= 512 bytes for Unicode) buffers.
     ;; Now we use exactly one buffer, or maybe two for package + symbol-name.
@@ -406,7 +403,7 @@
     ;; *READ-SUPPRESS* avoids creation of lists
     (test-reader-consing "#-sbcl(a (b c (d (e) (f) g)) h i j . x . y baz) 5"))
 (with-test (:name :read-symbol-does-not-cons-per-se
-            :fails-on (or :arm :ppc)) ; no idea why. does it vary by #+/-sb-thread?
+            :fails-on (or :mips :arm :ppc :sparc)) ; no idea why. does it vary by #+/-sb-thread?
   (test-reader-consing "COMMON-LISP-USER::A-SYMBOL"))
 (when (let ((s (find-symbol "ALLOCATE-VECTOR-ON-NUMBER-STACK" "SB-VM")))
         (and s (gethash s sb-c::*backend-parsed-vops*)))
@@ -612,3 +609,28 @@
       (assert-read-eqlity "0.333R0" 333/1000)
       (assert-read-eqlity "1R-3" 1/1000)
       (assert-read-eqlity ".1R2" 10))))
+
+(with-test (:name :stricter-sharp-o-etc)
+  ;; I think strictness is to be preferred. Compare to other implementations
+  ;; on the first example below of (read-from-string "#x a")
+  ;;   ABCL:  java.lang.StringIndexOutOfBoundsException: Index 0 out of bounds for length 0
+  ;;   CLISP: token "" after #x is not a rational number in base 16
+  ;;   ECL:   Cannot parse the #X readmacro."
+  ;; SHARP-O-FOOBAR writeup says this is all Undefined Behavior.
+  ;; https://www.lispworks.com/documentation/HyperSpec/Issues/iss316_w.htm
+  (assert-error (read-from-string "#x a")) ; used to read as 10.
+  (assert-error (read-from-string "#x#+foo a b")) ; used to skip A then read as 11.
+  (assert-error (read-from-string "#x #+foo a b")) ; same
+  ;; This was accepted because the nested READ return the rational number 1
+  ;; due to complex number canonicalization.
+  (handler-case (values (read-from-string "#o#c(1 0)"))
+    (condition (c) (assert (equal (simple-condition-format-arguments c) '(#\o 8 |#C|))))
+    (:no-error (x) (error "Expected an error but got ~A" x)))
+  ;; this was accepted because the nested READ returned rational 5
+  (handler-case (values (read-from-string "#o#.(floor 5.0)"))
+    (condition (c) (assert (equal (simple-condition-format-arguments c) '(#\o 8 |#.|))))
+    (:no-error (x) (error "Expected an error but got ~A" x)))
+  ;; This reads a token spelled "#." because #\# is a nonterminating macro, and #\. is a token
+  ;; constituent when not a dot by itself (or all dots). And #\( terminates a token.
+  ;; The token "#." is then ignored by *read-suppress*
+  (assert (equal (read-from-string "#+notfeat #o#.(progn wat)") '(progn wat))))
