@@ -50,7 +50,8 @@
                   (error "~@<Invalid evaluator mode: ~A. Must be one ~
                            of interpret, compile.~@:>"
                          mode)))))
-            ((string= arg "--break-on-failure")
+            ((or (string= arg "--break-on-failure")
+                 (string= arg "-b"))
              (setf *break-on-error* t)
              (setf test-util:*break-on-failure* t))
             ((string= arg "--break-on-expected-failure")
@@ -457,6 +458,8 @@
       (setf *input-manifest*
             (if manifest (read manifest) :ignore))))
   (format t "// Running pure tests (~a)~%" test-fun)
+  (when *break-on-error*
+    (enable-debugger))
   (let ((*failures* nil)
         ;; in case somebody corrupts CL-USER's use list, of course
         (standard-use-list (package-use-list "CL-USER")))
