@@ -1037,3 +1037,54 @@
               ((simple-string 9) a))
      (mismatch a b :start1 5 :end1 e))
    (or (integer 5 9) null)))
+
+(with-test (:name :reduce-append)
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :initial-value '(1)))
+    (('((a b c))) '(1 a b c) :test #'equal)
+    ((#((a b c))) '(1 a b c) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :initial-value '(1) :from-end t))
+    (('((a b c))) '(a b c 1) :test #'equal)
+    ((#((a b c))) '(a b c 1) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :end 1 :initial-value '(1)))
+    (('((a b c) (0))) '(1 a b c) :test #'equal)
+    ((#((a b c) (0))) '(1 a b c) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :end 1 :initial-value '(1) :from-end t))
+    (('((a b c) (0))) '(a b c 1) :test #'equal)
+    ((#((a b c) (0))) '(a b c 1) :test #'equal)))
+
+(with-test (:name :strings-in-sequences)
+  (assert-type
+   (lambda (n)
+     (let ((l '("a" "b")))
+       (dolist (e l)
+         (when (equal e n)
+           (return e)))))
+   (or null simple-string))
+  (assert-type
+   (lambda (n)
+     (aref #("a" "b") n))
+   simple-string))
+
+(with-test (:name :member-test-eq-reoptimize)
+  (assert (null
+           (ctu:ir1-named-calls
+            `(lambda (l m)
+               (unless l
+                 (and (member m (or l '(3 5 7)) :test #'eq) t))))))
+  (assert (null
+           (ctu:ir1-named-calls
+            `(lambda (l m)
+               (unless l
+                 (and (member m (or l '(3 5 7))) t)))))))
