@@ -1,7 +1,15 @@
-#+gc-stress
-(sb-thread:make-thread (lambda ()
-                         (loop (gc :full t) (sleep 0.001)))
-                       :name "gc stress")
+#+(and gc-stress (not gc-stress-delay))
+(progn
+  #+sb-thread
+  (sb-thread:make-thread (lambda ()
+                           (loop (gc :full t) (sleep 0.001)))
+                         :name "gc stress")
+  #-sb-thread
+  (sb-ext:schedule-timer (make-timer (lambda () (gc :full t))) 0.1 :repeat-interval 0.005))
+
+#+gc-verify
+(setf (sb-alien:extern-alien "verify_gens" char) 0
+      (extern-alien "pre_verify_gen_0" int) 1)
 
 (defpackage :test-util
   (:use :cl :sb-ext)
@@ -122,7 +130,6 @@
   #-sb-thread (error "can't make-kill-thread ~s" args)
   #+sb-thread
   (let ((thread (apply #'sb-thread:make-thread args)))
-    #-win32 ;; poor thread interruption on safepoints
     (when (boundp '*threads-to-kill*)
       (push thread *threads-to-kill*))
     thread))
@@ -460,6 +467,11 @@
      (list :filter (lambda (&key speed safety &allow-other-keys)
                      (and (> safety 0) (>= safety speed)))
            :compilation-speed 1 :space 1))
+    ((eql :safe-debug)
+     (list :filter (lambda (&key speed safety debug &allow-other-keys)
+                     (and (> safety 0) (>= safety speed)
+                          (> debug 1)))
+           :compilation-speed 1 :space 1))
     ((eql :quick)
      '(:compilation-speed 1 :space 1))
     ((eql :quick/incomplete)
@@ -691,18 +703,31 @@
           (multiple-value-bind (values conditions)
               (apply #'call-capturing-values-and-conditions function args)
             (typecase expected
-              ((cons (eql condition) (cons t null))
+              ((cons (eql condition))
                (let* ((expected-condition-type (second expected))
                       (unexpected (remove-if (lambda (condition)
                                                (typep condition
                                                       expected-condition-type))
                                              conditions))
+                      (test (third expected))
+                      (test-description (fourth expected))
                       (expected (set-difference conditions unexpected)))
                  (cond
                    (unexpected
                     (signaled-unexpected unexpected))
                    ((null expected)
-                    (failed-to-signal expected-condition-type)))))
+                    (failed-to-signal expected-condition-type)))
+                 (when (and test
+                            (not (every test conditions)))
+
+                   (error "~@<Calling the result of compiling~
+                      ~/test-util::print-form-and-optimize/ ~
+                      ~/test-util::print-arguments/~
+                      signaled unexpected condition~P~
+                      ~/test-util::print-signaled-conditions/~
+                      not matching~% ~a.~@:>"
+                          (cons form optimize) args (length conditions) conditions
+                          test-description))))
               (t
                (let ((expected (funcall expected)))
                  (cond
@@ -759,11 +784,13 @@
 ;;;
 ;;; If VALUES-FORM is of the form
 ;;;
-;;;   (CONDITION CONDITION-TYPE)
+;;;   (CONDITION CONDITION-TYPE &OPTIONAL FUNCTION)
 ;;;
 ;;; the function call is expected to signal the designated condition
 ;;; instead of returning values. CONDITION-TYPE is evaluated.
 ;;;
+;;; FUNCTION is called with CONDITION.
+;;;
 ;;; The OPTIMIZE keyword parameter controls the optimization policies
 ;;; (or policy) used when compiling FORM. The argument is interpreted
 ;;; as described for MAP-OPTIMIZE-DECLARATIONS*.
@@ -783,14 +810,14 @@
                (destructuring-bind (args values &key (test ''equal testp)
                                                      allow-conditions)
                    case
-                 (let ((conditionp (typep values '(cons (eql condition) (cons t null)))))
+                 (let ((conditionp (typep values '(cons (eql condition)))))
                    (when (and testp conditionp)
                      (sb-ext:with-current-source-form (case)
                        (error "~@<Cannot use ~S with ~S ~S.~@:>"
                               values :test test)))
                    `(list (lambda () (values ,@args))
                           ,(if conditionp
-                               `(list 'condition ,(second values))
+                               `(list 'condition ,(second values) ,(third values) ',(third values))
                                `(lambda () (multiple-value-list ,values)))
                           ,test
                           ,allow-conditions))))))
